######################################################################################################################################
#R code for analysis supporting the comment of Bertrand et al. (2020) on the work of Zellweger et al. (2020)
#Below is a R code of personal functions used in the technical comments.
#contact: romain.bertrand2@univ-tlse3.fr
######################################################################################################################################
# exporting coefficents
fixEff<- function(.) {
  summary(.)$coeff
}

#bootstrap model function adapted from the "bootMer" function (merTools package; version 1.11-1). The modification this function allows to save several model statistics for each boostrap iteration.
#see help ?bootMer in R for a detailed description of the function
bootMer2=function (x, FUN, nsim = 1, seed = NULL, use.u = FALSE, re.form = NA, 
                   type = c("parametric", "semiparametric"), verbose = FALSE, 
                   .progress = "none", PBargs = list(), parallel = c("no", "multicore", 
                                                                     "snow"), ncpus = getOption("boot.ncpus", 1L), cl = NULL) 
{
  stopifnot((nsim <- as.integer(nsim[1])) > 0)
  
  if (!is.null(seed)) 
    set.seed(seed)
  else if (!exists(".Random.seed", envir = .GlobalEnv)) 
    runif(1)
  mc <- match.call()
  t0 <- FUN(x)
  mle <- list(beta = getME(x, "beta"), theta = getME(x, "theta"))
  if (isLMM(x)) 
    mle <- c(mle, list(sigma = sigma(x)))
  if (type == "parametric") {
    argList <- list(x, nsim = nsim, na.action = na.exclude)
    if (!missing(re.form)) {
      argList <- c(argList, list(re.form = re.form)) #equivalent to use.u=F when re.form=NA
    }
    else {
      argList <- c(argList, list(use.u = use.u))
    }
    ss <- do.call(simulate, argList) 
  }
  else {
    if (!missing(re.form)) 
      stop(paste(sQuote("re.form")), "cannot be used with semiparametric bootstrapping")
    if (use.u) {
      if (isGLMM(x)) 
        warning("semiparametric bootstrapping is questionable for GLMMs")
      ss <- replicate(nsim, fitted(x) + sample(residuals(x, 
                                                         "response"), replace = TRUE), simplify = FALSE)
    }
    else {
      stop("semiparametric bootstrapping with use.u=FALSE not yet implemented")
    }
  }
  ffun <- local({
    FUN
    refit
    x
    ss
    length.t0 <- length(t0)
    f1 <- function(i) FUN(refit(x, ss[[i]])) #refit
    function(i) {
      ret <- f1(i)
      ret
    }
  })
  simvec <- seq_len(nsim)
  res <- parallel::mclapply(simvec, ffun, mc.cores = ncpus)
  res
}

#Computing R2 and coefficient values from boostrap model
statF=function(.){
  a=summary(.)$optinfo$conv$lme4$messages
  b=isSingular(.)[[1]]
  r2=r.squaredGLMM(.)
  if(is.null(a)==F){
    a=paste(a,sep="",collapse="_")
  }else{
    a=NA
  }
  d1=data.frame(summary(.)$varcor)
  return(list(r2=data.frame(r2m=r2[1,1],r2c=r2[1,2],warnM=as.character(a[[1]]),sing=b),coeff=data.frame(summary(.)$coeff,var=row.names(summary(.)$coeff)),randEff=as.data.frame(ranef(.,condVar=T)),sd_randI=data.frame(var=as.character(d1$grp),vcov=d1$vcov),varPart=data.frame(t(data.frame(calcVarPart(.))))))
}

